use std::env;
use std::path::Path;
use osu_db::listing::{Listing};
use sqlite::Value::{Integer, String as VString};


fn main() {
    let args: Vec<String> = env::args().collect();
    let game_directory = Path::new(&args[1]);

    let connection = sqlite::open("ranks.db").unwrap();
    connection.execute(
        "CREATE TABLE map (
            id INTEGER PRIMARY KEY,
            set_id INTEGER NOT NULL,
            mode INTEGER NOT NULL,
            name TEXT NOT NULL,

            length REAL NOT NULL,
            ranked INT NOT NULL,
            dmca INTEGER NOT NULL
        )"
    ).unwrap();

    // Avoid writing to disk since this is a one-time job
    connection.execute("PRAGMA synchronous=OFF").unwrap();
    connection.execute("PRAGMA count_changes=OFF").unwrap();
    connection.execute("PRAGMA journal_mode=MEMORY").unwrap();
    connection.execute("PRAGMA temp_store=MEMORY").unwrap();
    connection.execute("BEGIN TRANSACTION").unwrap();

    let mut insert = connection
        .prepare(
            "INSERT INTO map (
                id, set_id, mode, name, length, ranked, dmca
            ) VALUES (?, ?, ?, ?, ?, ?, 0)"
        )
        .unwrap()
        .cursor();

    let db_file = game_directory.join("osu!.db");
    let mut listing = Listing::from_file(db_file).unwrap();

    let nb_maps = listing.beatmaps.len();
    let mut i = 0;

    for beatmap in listing.beatmaps.iter_mut() {
        i = i + 1;

        let map_name = beatmap.title_ascii.as_ref().unwrap();
        let map_folder = beatmap.folder_name.as_ref().unwrap();

        println!("Processing beatmap {}/{} ({})", i, nb_maps, map_folder);
        match insert.bind(&[
            Integer(beatmap.beatmap_id as i64),
            Integer(beatmap.beatmapset_id as i64),
            Integer(beatmap.mode as i64),
            VString(map_name.to_string()),
            Integer(beatmap.drain_time as i64),
            Integer(beatmap.status.raw() as i64),
        ]) {
            Ok(_) => {},
            Err(e) => {
                // Some maps have duplicate ID, just ignore when that happens.
                println!("Failed to insert map into database: {}", e);
                continue;
            }
        };

        _ = insert.next();
    }

    connection.execute("COMMIT TRANSACTION").unwrap();
}
